<?php
/**
 * Copyright (C) 2014-2017 ServMask Inc.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ███████╗███████╗██████╗ ██╗   ██╗███╗   ███╗ █████╗ ███████╗██╗  ██╗
 * ██╔════╝██╔════╝██╔══██╗██║   ██║████╗ ████║██╔══██╗██╔════╝██║ ██╔╝
 * ███████╗█████╗  ██████╔╝██║   ██║██╔████╔██║███████║███████╗█████╔╝
 * ╚════██║██╔══╝  ██╔══██╗╚██╗ ██╔╝██║╚██╔╝██║██╔══██║╚════██║██╔═██╗
 * ███████║███████╗██║  ██║ ╚████╔╝ ██║ ╚═╝ ██║██║  ██║███████║██║  ██╗
 * ╚══════╝╚══════╝╚═╝  ╚═╝  ╚═══╝  ╚═╝     ╚═╝╚═╝  ╚═╝╚══════╝╚═╝  ╚═╝
 */

class Ai1wmfe_Settings_Controller {

	public static function index() {
		$model = new Ai1wmfe_Settings;

		$ftp_backup_schedules  = get_option( 'ai1wmfe_ftp_cron', array() );
		$last_backup_timestamp = get_option( 'ai1wmfe_ftp_timestamp', false );

		$last_backup_date = $model->get_last_backup_date( $last_backup_timestamp );
		$next_backup_date = $model->get_next_backup_date( $ftp_backup_schedules );

		Ai1wm_Template::render(
			'settings/index',
			array(
				'backups'              => get_option( 'ai1wmfe_ftp_backups', false ),
				'connection'           => get_option( 'ai1wmfe_ftp_connection', false ),
				'ftp_backup_schedules' => $ftp_backup_schedules,
				'directory'            => get_option( 'ai1wmfe_ftp_directory', false ),
				'email'                => get_option( 'ai1wmfe_ftp_notify_email', get_option( 'admin_email', '' ) ),
				'hostname'             => get_option( 'ai1wmfe_ftp_hostname', false ),
				'notify'               => get_option( 'ai1wmfe_ftp_notify_toggle', false ),
				'last_backup_date'     => $last_backup_date,
				'next_backup_date'     => $next_backup_date,
				'passive'              => get_option( 'ai1wmfe_ftp_passive', false ),
				'authentication'       => get_option( 'ai1wmfe_ftp_authentication', 'password' ),
				'password'             => get_option( 'ai1wmfe_ftp_password', false ),
				'key'                  => get_option( 'ai1wmfe_ftp_key', false ),
				'passphrase'           => get_option( 'ai1wmfe_ftp_passphrase', false ),
				'port'                 => get_option( 'ai1wmfe_ftp_port', AI1WMFE_FTP_DEFAULT_PORT ),
				'total'                => get_option( 'ai1wmfe_ftp_total', false ),
				'username'             => get_option( 'ai1wmfe_ftp_username', false ),
				'type'                 => get_option( 'ai1wmfe_ftp_type', 'ftp' ),
			),
			AI1WMFE_TEMPLATES_PATH
		);
	}

	public static function connection() {
		$files  = stripslashes_deep( $_FILES );
		$params = stripslashes_deep( $_POST );

		// Test FTP connection
		if ( isset( $params['ai1wmfe_ftp_update'] ) ) {

			// Type
			if ( isset( $params['ai1wmfe_ftp_type'] ) ) {
				update_option( 'ai1wmfe_ftp_type', trim( $params['ai1wmfe_ftp_type'] ) );
			}

			// Hostname
			if ( isset( $params['ai1wmfe_ftp_hostname'] ) ) {
				update_option( 'ai1wmfe_ftp_hostname', trim( $params['ai1wmfe_ftp_hostname'] ) );
			}

			// Authentication
			if ( isset( $params['ai1wmfe_ftp_authentication'] ) ) {
				update_option( 'ai1wmfe_ftp_authentication', trim( $params['ai1wmfe_ftp_authentication'] ) );
			}

			// Username
			if ( isset( $params['ai1wmfe_ftp_username'] ) ) {
				update_option( 'ai1wmfe_ftp_username', trim( $params['ai1wmfe_ftp_username'] ) );
			}

			// Password
			if ( ! empty( $params['ai1wmfe_ftp_password'] ) ) {
				update_option( 'ai1wmfe_ftp_password', trim( $params['ai1wmfe_ftp_password'] ) );
			}

			// Private key
			if ( is_uploaded_file( $files['ai1wmfe_ftp_key']['tmp_name'] ) ) {
				update_option( 'ai1wmfe_ftp_key', file_get_contents( $files['ai1wmfe_ftp_key']['tmp_name'] ) );

				// Unset empty password
				if ( empty( $params['ai1wmfe_ftp_password'] ) ) {
					update_option( 'ai1wmfe_ftp_password', trim( $params['ai1wmfe_ftp_password'] ) );
				}
			}

			// Passphrase
			if ( ! empty( $params['ai1wmfe_ftp_passphrase'] ) ) {
				update_option( 'ai1wmfe_ftp_passphrase', trim( $params['ai1wmfe_ftp_passphrase'] ) );
			}

			// Directory
			if ( isset( $params['ai1wmfe_ftp_directory'] ) ) {
				update_option( 'ai1wmfe_ftp_directory', trim( $params['ai1wmfe_ftp_directory'] ) );
			}

			// Port
			if ( isset( $params['ai1wmfe_ftp_port'] ) ) {
				update_option( 'ai1wmfe_ftp_port', intval( $params['ai1wmfe_ftp_port'] ) );
			}

			// Passive Mode
			if ( isset( $params['ai1wmfe_ftp_passive'] ) ) {
				update_option( 'ai1wmfe_ftp_passive', 1 );
			} else {
				update_option( 'ai1wmfe_ftp_passive', 0 );
			}

			// Set client
			$ftp = ServMaskClientFactory::create(
				get_option( 'ai1wmfe_ftp_type' ),
				get_option( 'ai1wmfe_ftp_hostname' ),
				get_option( 'ai1wmfe_ftp_authentication' ),
				get_option( 'ai1wmfe_ftp_username' ),
				get_option( 'ai1wmfe_ftp_password' ),
				get_option( 'ai1wmfe_ftp_key' ),
				get_option( 'ai1wmfe_ftp_passphrase' ),
				get_option( 'ai1wmfe_ftp_directory' ),
				get_option( 'ai1wmfe_ftp_port' ),
				get_option( 'ai1wmfe_ftp_passive' )
			);

			// Test FTP connection
			update_option( 'ai1wmfe_ftp_connection', (int) $ftp->testConnection() );

			// FTP append file
			update_option( 'ai1wmfe_ftp_append', (int) $ftp->testAppendFile() );
		}

		// Redirect to settings page
		wp_redirect( add_query_arg( 'message', 1, network_admin_url( 'admin.php?page=site-migration-ftp-settings' ) ) );
		exit;
	}

	public static function settings() {
		$params = stripslashes_deep( $_POST );

		// FTP update
		if ( isset( $params['ai1wmfe_ftp_update'] ) ) {
			$model = new Ai1wmfe_Settings;

			// Cron update
			if ( isset( $params['ai1wmfe_ftp_cron'] ) ) {
				$model->set_cron( (array) $params['ai1wmfe_ftp_cron'] );
			} else {
				$model->set_cron( array() );
			}

			// Set number of backups
			if ( ! empty( $params['ai1wmfe_ftp_backups'] ) ) {
				$model->set_backups( (int) $params['ai1wmfe_ftp_backups'] );
			} else {
				$model->set_backups( 0 );
			}

			// Set size of backups
			if ( ! empty( $params['ai1wmfe_ftp_total'] ) && ! empty( $params['ai1wmfe_ftp_total_unit'] ) ) {
				$model->set_total( (int) $params['ai1wmfe_ftp_total'] . trim( $params['ai1wmfe_ftp_total_unit'] ) );
			} else {
				$model->set_total( 0 );
			}

			// Set notification toggle
			$model->set_toggle( isset( $params['ai1wmfe_notification_toggle'] ) );

			// Set notification email
			$model->set_email( $params['ai1wmfe_notification_email'] );
		}

		// Redirect to settings page
		wp_redirect( network_admin_url( 'admin.php?page=site-migration-ftp-settings' ) );
		exit;
	}
}
