<?php
/**
 * Copyright (C) 2014-2017 ServMask Inc.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ███████╗███████╗██████╗ ██╗   ██╗███╗   ███╗ █████╗ ███████╗██╗  ██╗
 * ██╔════╝██╔════╝██╔══██╗██║   ██║████╗ ████║██╔══██╗██╔════╝██║ ██╔╝
 * ███████╗█████╗  ██████╔╝██║   ██║██╔████╔██║███████║███████╗█████╔╝
 * ╚════██║██╔══╝  ██╔══██╗╚██╗ ██╔╝██║╚██╔╝██║██╔══██║╚════██║██╔═██╗
 * ███████║███████╗██║  ██║ ╚████╔╝ ██║ ╚═╝ ██║██║  ██║███████║██║  ██╗
 * ╚══════╝╚══════╝╚═╝  ╚═╝  ╚═══╝  ╚═╝     ╚═╝╚═╝  ╚═╝╚══════╝╚═╝  ╚═╝
 */

class Ai1wmfe_Settings {

	public function get_last_backup_date( $last_backup_timestamp ) {
		if ( $last_backup_timestamp ) {
			$last_backup_date = get_date_from_gmt( date( 'Y-m-d H:i:s', $last_backup_timestamp ), 'F j, Y g:i a' );
		} else {
			$last_backup_date = __( 'None', AI1WMFE_PLUGIN_NAME );
		}

		return $last_backup_date;
	}

	public function get_next_backup_date( $schedules ) {
		$future_backup_timestamps = array();

		foreach ( $schedules as $schedule ) {
			$future_backup_timestamps[] = wp_next_scheduled( "ai1wmfe_ftp_{$schedule}_export", array( array(
					'secret_key' => get_option( AI1WM_SECRET_KEY ),
					'ftp'        => 1,
				),
			) );
		}

		sort( $future_backup_timestamps );

		if ( isset( $future_backup_timestamps[0] ) ) {
			$next_backup_date = get_date_from_gmt( date( 'Y-m-d H:i:s', $future_backup_timestamps[0] ), 'F j, Y g:i a' );
		} else {
			$next_backup_date = __( 'None', AI1WMFE_PLUGIN_NAME );
		}

		return $next_backup_date;
	}

	public function set_cron( $schedules ) {
		// Reset cron schedules
		Ai1wm_Cron::clear( 'ai1wmfe_ftp_hourly_export' );
		Ai1wm_Cron::clear( 'ai1wmfe_ftp_daily_export' );
		Ai1wm_Cron::clear( 'ai1wmfe_ftp_weekly_export' );
		Ai1wm_Cron::clear( 'ai1wmfe_ftp_monthly_export' );

		// Update cron options
		update_option( 'ai1wmfe_ftp_cron', $schedules );

		// Update cron schedules
		foreach ( $schedules as $schedule ) {
			Ai1wm_Cron::add( "ai1wmfe_ftp_{$schedule}_export", $schedule, array(
				'secret_key' => get_option( AI1WM_SECRET_KEY ),
				'ftp'        => 1,
			) );
		}
	}

	public function set_backups( $number ) {
		update_option( 'ai1wmfe_ftp_backups', $number );
	}

	public function set_total( $size ) {
		update_option( 'ai1wmfe_ftp_total', $size );
	}

	public function set_email( $email ) {
		update_option( 'ai1wmfe_ftp_notify_email', $email );
	}

	public function set_toggle( $toggle ) {
		update_option( 'ai1wmfe_ftp_notify_toggle', $toggle );
	}
}
