<?php
/**
 * Copyright (C) 2014-2017 ServMask Inc.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ███████╗███████╗██████╗ ██╗   ██╗███╗   ███╗ █████╗ ███████╗██╗  ██╗
 * ██╔════╝██╔════╝██╔══██╗██║   ██║████╗ ████║██╔══██╗██╔════╝██║ ██╔╝
 * ███████╗█████╗  ██████╔╝██║   ██║██╔████╔██║███████║███████╗█████╔╝
 * ╚════██║██╔══╝  ██╔══██╗╚██╗ ██╔╝██║╚██╔╝██║██╔══██║╚════██║██╔═██╗
 * ███████║███████╗██║  ██║ ╚████╔╝ ██║ ╚═╝ ██║██║  ██║███████║██║  ██╗
 * ╚══════╝╚══════╝╚═╝  ╚═╝  ╚═══╝  ╚═╝     ╚═╝╚═╝  ╚═╝╚══════╝╚═╝  ╚═╝
 */

class Ai1wmfe_Export_Upload {

	public static function execute( $params ) {

		// Set completed flag
		$params['completed'] = false;

		// Set offset
		if ( ! isset( $params['offset'] ) ) {
			$params['offset'] = 0;
		}

		// Set retry
		if ( ! isset( $params['retry'] ) ) {
			$params['retry'] = 0;
		}

		// Set client
		$ftp = ServMaskClientFactory::create(
			get_option( 'ai1wmfe_ftp_type' ),
			get_option( 'ai1wmfe_ftp_hostname' ),
			get_option( 'ai1wmfe_ftp_authentication' ),
			get_option( 'ai1wmfe_ftp_username' ),
			get_option( 'ai1wmfe_ftp_password' ),
			get_option( 'ai1wmfe_ftp_key' ),
			get_option( 'ai1wmfe_ftp_passphrase' ),
			get_option( 'ai1wmfe_ftp_directory' ),
			get_option( 'ai1wmfe_ftp_port' ),
			get_option( 'ai1wmfe_ftp_passive' )
		);

		// Get archive file
		$archive = fopen( ai1wm_archive_path( $params ), 'rb' );

		try {

			// Increase number of retries
			$params['retry'] += 1;

			// Upload file chunk or upload file
			if ( get_option( 'ai1wmfe_ftp_append', false ) ) {
				// Upload file chunk
				$ftp->uploadFileChunk(
					sprintf( '%s/%s', ai1wm_archive_folder(), ai1wm_archive_name( $params ) ),
					$archive,
					$params
				);
			} else {
				// Upload file depends on the connection
				$ftp->uploadFile(
					sprintf( '%s/%s', ai1wm_archive_folder(), ai1wm_archive_name( $params ) ),
					$archive,
					$params
				);
			}
		} catch ( Exception $e ) {
			// Retry 3 times
			if ( $params['retry'] <= 3 ) {
				return $params;
			}

			throw $e;
		}

		// Unset retry counter
		unset( $params['retry'] );

		// Set archive details
		$name  = ai1wm_archive_name( $params );
		$bytes = ai1wm_archive_bytes( $params );
		$size  = ai1wm_archive_size( $params );

		// Upload completed?
		if ( $params['offset'] < $bytes ) {

			// Get progress
			if ( isset( $params['offset'] ) ) {
				$progress = (int) ( ( $params['offset'] / $bytes ) * 100 );
			} else {
				$progress = 100;
			}

			// Set progress
			Ai1wm_Status::info(
				sprintf(
					__(
						'<i class="ai1wm-icon-ftp"></i> ' .
						'Uploading <strong>%s</strong> (%s)<br />%d%% complete',
						AI1WMFE_PLUGIN_NAME
					),
					$name,
					$size,
					$progress
				)
			);

		} else {

			// Set last backup date
			update_option( 'ai1wmfe_ftp_timestamp', time() );

			// Set progress
			if ( get_option( 'ai1wmfe_ftp_type' ) === 'sftp' ) {
				Ai1wm_Status::done(
					__( 'Your WordPress archive has been uploaded to the SFTP server.', AI1WMFE_PLUGIN_NAME ),
					__( 'SFTP', AI1WMFE_PLUGIN_NAME )
				);
			} else {
				Ai1wm_Status::done(
					__( 'Your WordPress archive has been uploaded to the FTP server.', AI1WMFE_PLUGIN_NAME ),
					__( 'FTP', AI1WMFE_PLUGIN_NAME )
				);
			}

			self::notify_admin_of_new_backup( $params );

			// Unset offset
			unset( $params['offset'] );

			// Unset completed flag
			unset( $params['completed'] );
		}

		// Close archive
		fclose( $archive );

		return $params;
	}

	public static function is_notification_enabled( $recipient ) {
		return (
			get_option( 'ai1wmfe_ftp_notify_toggle', false ) &&
			! empty( $recipient ) &&
			function_exists( 'wp_mail' )
		);
	}

	public static function notify_admin_of_new_backup( $params ) {
		$recipient = get_option( 'ai1wmfe_ftp_notify_email', get_option( 'admin_email', '' ) );
		if ( self::is_notification_enabled( $recipient ) ) {

			if ( get_option( 'ai1wmfe_ftp_type' ) === 'sftp' ) {
				$subject = __( 'SFTP Backup report', AI1WMFE_PLUGIN_NAME );
				$message = sprintf( __( "Your site %s was successfully exported to SFTP.\n", AI1WMFE_PLUGIN_NAME ), site_url() );
			} else {
				$subject = __( 'FTP Backup report', AI1WMFE_PLUGIN_NAME );
				$message = sprintf( __( "Your site %s was successfully exported to FTP.\n", AI1WMFE_PLUGIN_NAME ), site_url() );
			}

			$message .= "\n\n";
			$message .= sprintf( __( "Date: %s\n", AI1WMFE_PLUGIN_NAME ), date( 'r' ) );
			$message .= sprintf( __( "Backup file: %s\n", AI1WMFE_PLUGIN_NAME ), ai1wm_archive_name( $params ) );
			$message .= sprintf( __( "Size: %s\n", AI1WMFE_PLUGIN_NAME ), ai1wm_archive_size( $params ) );
			wp_mail( $recipient, $subject, $message );
		}
	}
}
