<?php
/**
 * Copyright (C) 2014-2017 ServMask Inc.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ███████╗███████╗██████╗ ██╗   ██╗███╗   ███╗ █████╗ ███████╗██╗  ██╗
 * ██╔════╝██╔════╝██╔══██╗██║   ██║████╗ ████║██╔══██╗██╔════╝██║ ██╔╝
 * ███████╗█████╗  ██████╔╝██║   ██║██╔████╔██║███████║███████╗█████╔╝
 * ╚════██║██╔══╝  ██╔══██╗╚██╗ ██╔╝██║╚██╔╝██║██╔══██║╚════██║██╔═██╗
 * ███████║███████╗██║  ██║ ╚████╔╝ ██║ ╚═╝ ██║██║  ██║███████║██║  ██╗
 * ╚══════╝╚══════╝╚═╝  ╚═╝  ╚═══╝  ╚═╝     ╚═╝╚═╝  ╚═╝╚══════╝╚═╝  ╚═╝
 */

class Ai1wmfe_Import_Download {

	public static function execute( $params ) {

		// Set completed flag
		$params['completed'] = false;

		// Total bytes
		if ( ! isset( $params['totalBytes'] ) ) {
			throw new Ai1wm_Import_Exception(
				__( 'Unable to determine file size of FTP File.', AI1WMFE_PLUGIN_NAME )
			);
		}

		// Set offset
		$params['offset'] = ai1wm_archive_bytes( $params );

		// Set retry
		if ( ! isset( $params['retry'] ) ) {
			$params['retry'] = 0;
		}

		// Set client
		$ftp = ServMaskClientFactory::create(
			get_option( 'ai1wmfe_ftp_type' ),
			get_option( 'ai1wmfe_ftp_hostname' ),
			get_option( 'ai1wmfe_ftp_authentication' ),
			get_option( 'ai1wmfe_ftp_username' ),
			get_option( 'ai1wmfe_ftp_password' ),
			get_option( 'ai1wmfe_ftp_key' ),
			get_option( 'ai1wmfe_ftp_passphrase' ),
			get_option( 'ai1wmfe_ftp_directory' ),
			get_option( 'ai1wmfe_ftp_port' ),
			get_option( 'ai1wmfe_ftp_passive' )
		);

		// Get archive file
		$archive = fopen( ai1wm_archive_path( $params ), 'ab' );

		try {

			// Increase number of retries
			$params['retry'] += 1;

			// Download file chunk or download file
			if ( get_option( 'ai1wmfe_ftp_append', false ) ) {
				$ftp->downloadFileChunk( $params['filePath'], $archive, $params );
			} else {
				$ftp->downloadFile( $params['filePath'], $archive, $params );
			}
		} catch ( Exception $e ) {
			// Retry 3 times
			if ( $params['retry'] <= 3 ) {
				return $params;
			}

			throw $e;
		}

		// Unset retry counter
		unset( $params['retry'] );

		// Calculate percent
		$percent = (int) ( ( $params['offset'] / $params['totalBytes'] ) * 100 );

		// Set progress
		Ai1wm_Status::progress( $percent );

		// Completed?
		if ( $params['totalBytes'] == $params['offset'] ) {

			// Unset offset
			unset( $params['offset'] );

			// Unset completed flag
			unset( $params['completed'] );

		}

		// Close the archive file
		fclose( $archive );

		return $params;
	}
}
