<?php
/**
 * Copyright (C) 2014-2017 ServMask Inc.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ███████╗███████╗██████╗ ██╗   ██╗███╗   ███╗ █████╗ ███████╗██╗  ██╗
 * ██╔════╝██╔════╝██╔══██╗██║   ██║████╗ ████║██╔══██╗██╔════╝██║ ██╔╝
 * ███████╗█████╗  ██████╔╝██║   ██║██╔████╔██║███████║███████╗█████╔╝
 * ╚════██║██╔══╝  ██╔══██╗╚██╗ ██╔╝██║╚██╔╝██║██╔══██║╚════██║██╔═██╗
 * ███████║███████╗██║  ██║ ╚████╔╝ ██║ ╚═╝ ██║██║  ██║███████║██║  ██╗
 * ╚══════╝╚══════╝╚═╝  ╚═╝  ╚═══╝  ╚═╝     ╚═╝╚═╝  ╚═╝╚══════╝╚═╝  ╚═╝
 */

class ServMaskClientFactory
{
	/**
	 * Create the correct client depending on the type
	 *
	 * @param  string  $type           Connection type.
	 * @param  string  $hostname       Server hostname.
	 * @param  string  $authentication Client authentication.
	 * @param  string  $username       Client username.
	 * @param  string  $password       Client password.
	 * @param  string  $key            Client private key.
	 * @param  string  $passphrase     Client private key passphrase.
	 * @param  string  $directory      Server directory.
	 * @param  integer $port           Server port.
	 * @param  boolean $passive        Passive mode.
	 * @return object                  The newly created client.
	 */
	public static function create( $type, $hostname, $authentication, $username, $password, $key, $passphrase, $directory, $port, $passive ) {
		if ( $type === 'ftp' ) {
			return new ServMaskFTPClient( $hostname, $username, $password, $directory, $port, $passive, false );
		} else if ( $type === 'ftps' ) {
			return new ServMaskFTPClient( $hostname, $username, $password, $directory, $port, $passive, true );
		} else if ( $type === 'sftp' ) {
			return new ServMaskSFTPClient( $hostname, $authentication, $username, $password, $key, $passphrase, $directory, $port );
		} else {
			return new Exception( sprintf( 'You have supplied an invalid factory type = %s', $type ) );
		}
	}
}
