<?php
/**
 * Copyright (C) 2014-2017 ServMask Inc.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ███████╗███████╗██████╗ ██╗   ██╗███╗   ███╗ █████╗ ███████╗██╗  ██╗
 * ██╔════╝██╔════╝██╔══██╗██║   ██║████╗ ████║██╔══██╗██╔════╝██║ ██╔╝
 * ███████╗█████╗  ██████╔╝██║   ██║██╔████╔██║███████║███████╗█████╔╝
 * ╚════██║██╔══╝  ██╔══██╗╚██╗ ██╔╝██║╚██╔╝██║██╔══██║╚════██║██╔═██╗
 * ███████║███████╗██║  ██║ ╚████╔╝ ██║ ╚═╝ ██║██║  ██║███████║██║  ██╗
 * ╚══════╝╚══════╝╚═╝  ╚═╝  ╚═══╝  ╚═╝     ╚═╝╚═╝  ╚═╝╚══════╝╚═╝  ╚═╝
 */

interface ServMaskFTPInterface
{
	/**
	 * Set hostname
	 *
	 * @param  string               $value FTP hostname
	 * @return ServMaskFTPInterface
	 */
	public function setHostname( $value );

	/**
	 * Get hostname
	 *
	 * @return string
	 */
	public function getHostname();

	/**
	 * Set username
	 *
	 * @param  string               $username FTP Username
	 * @return ServMaskFTPInterface
	 */
	public function setUsername( $value );

	/**
	 * Get username
	 *
	 * @return string
	 */
	public function getUsername();

	/**
	 * Set password
	 *
	 * @param  string               $value FTP Password
	 * @return ServMaskFTPInterface
	 */
	public function setPassword( $value );

	/**
	 * Get password
	 *
	 * @return string
	 */
	public function getPassword();

	/**
	 * Set directory
	 *
	 * @param  string               $value FTP Directory
	 * @return ServMaskFTPInterface
	 */
	public function setDirectory( $value );

	/**
	 * Get directory
	 *
	 * @return string
	 */
	public function getDirectory();

	/**
	 * Set port
	 *
	 * @param  integer              $value FTP Port
	 * @return ServMaskFTPInterface
	 */
	public function setPort( $value );

	/**
	 * Get port
	 *
	 * @return integer
	 */
	public function getPort();

	/**
	 * Set passive mode
	 *
	 * @param  boolean              $value FTP Passive Mode
	 * @return ServMaskFTPInterface
	 */
	public function setPassive( $value );

	/**
	 * Get passive mode
	 *
	 * @return boolean
	 */
	public function getPassive();

	/**
	 * Create folder
	 *
	 * @param  string  $path FTP path
	 * @return boolean
	 */
	public function createFolder( $path );

	/**
	 * Retrieve files and folders metadata
	 *
	 * @param  string $path FTP path
	 * @return array
	 */
	public function listFolder( $path );

	/**
	 * Retrieve files and folders raw metadata
	 *
	 * @param  string $path FTP path
	 * @return array
	 */
	public function rawListFolder( $path );

	/**
	 * Upload file
	 *
	 * @param  string $remoteFile FTP path
	 * @param  mixed  $localFile  File path or file stream
	 * @param  array  $params     File parameters
	 * @return array
	 */
	public function uploadFile( $remoteFile, $localFile, &$params = array() );

	/**
	 * Upload file chunk
	 *
	 * @param  string $remoteFile FTP path
	 * @param  mixed  $localFile  File path or file stream
	 * @param  array  $params     File parameters
	 * @return array
	 */
	public function uploadFileChunk( $remoteFile, $localFile, &$params = array() );

	/**
	 * Download file
	 *
	 * @param  string $remoteFile FTP path
	 * @param  mixed  $localFile  File path or file stream
	 * @param  array  $params     File parameters
	 * @return array
	 */
	public function downloadFile( $remoteFile, $localFile, &$params = array() );

	/**
	 * Download file chunk
	 *
	 * @param  string $remoteFile FTP path
	 * @param  mixed  $localFile  File path or file stream
	 * @param  array  $params     File parameters
	 * @return array
	 */
	public function downloadFileChunk( $remoteFile, $localFile, &$params = array() );

	/**
	 * Remove file
	 *
	 * @param  string  $path FTP path
	 * @return boolean
	 */
	public function removeFile( $path );

	/**
	 * Test FTP connection
	 *
	 * @return boolean
	 */
	public function testConnection();

	/**
	 * Test FTP append file
	 *
	 * @return boolean
	 */
	public function testAppendFile( $remoteFile = '.servmask' );
}
